"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FlexibleObject = void 0;
class FlexibleObject {
    #parent;
    #paths;
    #data;
    #undefined = true;
    #markFlexibleObjectDefined() {
        if (!this.#undefined)
            return;
        this.#undefined = false;
        if (this.#parent)
            this.#parent.#markFlexibleObjectDefined();
    }
    constructor(data, parent, paths) {
        this.#parent = parent;
        this.#data = {};
        this.#paths = paths ? paths : [];
        if (data !== undefined)
            this.setValue(data);
        return new Proxy(this, {
            get: (target, p) => {
                if (this[p])
                    return (...args) => this[p](...args);
                if (target.#data[p] === undefined)
                    target.#data[p] = new FlexibleObject(target.#data[p], this, [...this.#paths, p]);
                return target.#data[p];
            },
            set: (target, p, newValue) => {
                return false;
            }
        });
    }
    /**
     * Set object node value
     * @param value
     */
    // @ts-ignore
    setValue(value) {
        if (typeof value === 'object' && !Array.isArray(value)) {
            Object.keys(value).forEach(key => {
                this.#data[key] = new FlexibleObject(this.#data[key], this, [...this.#paths, key]);
                this.#data[key].setValue(value[key]);
            });
        }
        else {
            this.#data = value;
        }
        this.#markFlexibleObjectDefined();
    }
    /**
     * Get object node value
     * @param defaultValue
     * @param onUndefinedCallback
     * @param getPathIndex
     */
    // @ts-ignore
    getValue(defaultValue, onUndefinedCallback, getPathIndex) {
        if (this.#undefined) {
            if (onUndefinedCallback)
                onUndefinedCallback(this.getPath(getPathIndex));
            if (defaultValue !== undefined) {
                return defaultValue;
            }
            return undefined;
        }
        if (typeof this.#data === 'object' && !Array.isArray(this.#data)) {
            const dumpObject = {};
            Object.keys(this.#data).forEach((key) => {
                const dumpResult = this.#data[key].getValue();
                if (dumpResult === undefined)
                    return;
                dumpObject[key] = dumpResult;
            });
            return dumpObject;
        }
        else {
            return this.#undefined ? undefined : this.#data;
        }
    }
    /**
     * Is current object's value undefined
     */
    // @ts-ignore
    isUndefined() {
        return this.#undefined;
    }
    /**
     * Get current object path
     * The index parameter indicates array item if this object is an array
     * @param index
     */
    // @ts-ignore
    getPath(index) {
        const objPath = this.#paths.join('.');
        if (index === undefined)
            return objPath;
        return `${objPath}[${index}]`;
    }
}
exports.FlexibleObject = FlexibleObject;
